@@ -17,6 +17,7 @@
 */
 
 #include "kcmdlineargs.h"
+#include <kdebug.h>
 
 #include <config.h>
 
@@ -43,7 +44,6 @@
 #include "kdeversion.h"
 #include "kcomponentdata.h"
 #include "kglobal.h"
-#include "kstringhandler.h"
 #include "kurl.h"
 
 #include "kuitsemantics_p.h" // for escaping arguments in i18n
@@ -167,8 +167,9 @@ class KCmdLineArgsStatic {
     KCmdLineArgsList *argsList; // All options.
     const KAboutData *about;
 
-    int argc; // The original argc
-    char **argv; // The original argv
+    int all_argc; // The original argc
+    char **all_argv; // The original argv
+    char *appName;
     bool parsed : 1; // Whether we have parsed the arguments since calling init
     bool ignoreUnknown : 1; // Ignore unknown options and arguments
     QByteArray mCwd; // Current working directory. Important for KUnqiueApp!
@@ -254,8 +255,9 @@ K_GLOBAL_STATIC(KCmdLineArgsStatic, s)
 KCmdLineArgsStatic::KCmdLineArgsStatic () {
     // Global data
     argsList = 0;
-    argc = 0;
-    argv = 0;
+    all_argc = 0;
+    all_argv = 0;
+    appName = 0;
     mCwd.clear();
     about = 0;
     parsed = false;
@@ -264,7 +266,6 @@ KCmdLineArgsStatic::KCmdLineArgsStatic () {
 
     // Text codec.
     codec = QTextCodec::codecForLocale();
-    setlocale(LC_ALL, ""); // need to initialize "System" codec, i.e. iconv
 
     // Qt options
     //FIXME: Check if other options are specific to Qt/X11
@@ -290,6 +291,7 @@ KCmdLineArgsStatic::KCmdLineArgsStatic () {
     qt_options.add("button <color>", ki18n("sets the default button color"));
     qt_options.add("name <name>", ki18n("sets the application name"));
     qt_options.add("title <title>", ki18n("sets the application title (caption)"));
+    qt_options.add("testability", ki18n("load the testability framework"));
 #ifdef Q_WS_X11
     qt_options.add("visual TrueColor", ki18n("forces the application to use a TrueColor visual on\nan 8-bit display"));
     qt_options.add("inputstyle <inputstyle>", ki18n("sets XIM (X Input Method) input style. Possible\nvalues are onthespot, overthespot, offthespot and\nroot"));
@@ -301,11 +303,7 @@ KCmdLineArgsStatic::KCmdLineArgsStatic () {
 #endif
     qt_options.add("reverse", ki18n("mirrors the whole layout of widgets"));
     qt_options.add("stylesheet <file.qss>", ki18n("applies the Qt stylesheet to the application widgets"));
-#if QT_VERSION >= 0x040a00
-# error Qt version larger than 4.9, please fix me
-#endif
-    if (qVersion()[2] >= '5')
-        qt_options.add("graphicssystem <system>", ki18n("use a different graphics system instead of the default one, options are raster and opengl (experimental)"));
+    qt_options.add("graphicssystem <system>", ki18n("use a different graphics system instead of the default one, options are raster and opengl (experimental)"));
     // KDE options
     kde_options.add("caption <caption>",   ki18n("Use 'caption' as name in the titlebar"));
     kde_options.add("icon <icon>",         ki18n("Use 'icon' as the application icon"));
@@ -315,7 +313,7 @@ KCmdLineArgsStatic::KCmdLineArgsStatic () {
     kde_options.add("waitforwm",           ki18n("Waits for a WM_NET compatible windowmanager"));
 #endif
     kde_options.add("style <style>",       ki18n("sets the application GUI style"));
-    kde_options.add("geometry <geometry>", ki18n("sets the client geometry of the main widget - see man X for the argument format"));
+    kde_options.add("geometry <geometry>", ki18n("sets the client geometry of the main widget - see man X for the argument format (usually WidthxHeight+XPos+YPos)"));
 #ifndef Q_WS_WIN
     kde_options.add("smkey <sessionKey>"); // this option is obsolete and exists only to allow smooth upgrades from sessions
 #endif
@@ -343,7 +341,8 @@ public:
         , parsedOptionList(0)
         , parsedArgList(0)
         , isQt(id == "qt")
-    {}
+    {
+    }
     ~KCmdLineArgsPrivate()
     {
         delete parsedOptionList;
@@ -448,10 +447,10 @@ KCmdLineArgs::init(const KAboutData* ab)
 void
 KCmdLineArgs::init(int _argc, char **_argv, const KAboutData *_about, StdCmdLineArgs stdargs)
 {
-   s->argc = _argc;
-   s->argv = _argv;
+   s->all_argc = _argc;
+   s->all_argv = _argv;
 
-   if (!s->argv)
+   if (!s->all_argv)
    {
       fprintf(stderr, "\n\nFAILURE (KCmdLineArgs):\n");
       fprintf(stderr, "Passing null-pointer to 'argv' is not allowed.\n\n");
@@ -461,10 +460,12 @@ KCmdLineArgs::init(int _argc, char **_argv, const KAboutData *_about, StdCmdLine
    }
 
    // Strip path from argv[0]
-   if (s->argc) {
-     char *p = strrchr( s->argv[0], '/');
+   if (s->all_argc) {
+     char *p = strrchr(s->all_argv[0], QDir::separator().toAscii());
      if (p)
-       s->argv[0] = p+1;
+       s->appName = p+1;
+     else
+       s->appName = s->all_argv[0];
    }
 
    s->about = _about;
@@ -480,8 +481,8 @@ QString KCmdLineArgs::cwd()
 
 QString KCmdLineArgs::appName()
 {
-   if (!s->argc) return QString();
-   return s->decodeInput(s->argv[0]);
+   if (!s->appName) return QString();
+   return s->decodeInput(s->appName);
 }
 
 /**
@@ -513,8 +514,9 @@ KCmdLineArgs::addCmdLineOptions( const KCmdLineOptions &options, const KLocalize
    int i = 0;
    for(args = s->argsList->begin(); args != s->argsList->end(); ++args, i++)
    {
-      if (id == (*args)->d->id)
+       if (id == (*args)->d->id) {
          return; // Options already present.
+      }
 
       // Only check for afterId if it has been given non-empty, as the
       // unnamed option group should come after all named groups.
@@ -587,14 +589,20 @@ KCmdLineArgs::loadAppArgs( QDataStream &ds)
      QByteArray id;
      ds >> id;
      Q_ASSERT( s->argsList );
+     bool found = false;
      for(args = s->argsList->begin(); args != s->argsList->end(); ++args)
      {
        if ((*args)->d->id  == id)
        {
           (*args)->d->load(ds);
+          found = true;
           break;
        }
      }
+     if (!found) {
+         kWarning() << "Argument definitions for" << id << "not found!";
+         // The next ds >> id will do nonsensical things...
+     }
    }
    s->parsed = true;
 }
@@ -662,7 +670,7 @@ KCmdLineArgsStatic::findOption(const KCmdLineOptions &options, QByteArray &opt,
          opt_name = opt_name.mid(1);
          result = 4;
       }
-      if (opt_name.startsWith("no"))
+      if (opt_name.startsWith("no") && !opt_name.contains('<')) // krazy:exclude=strings
       {
          opt_name = opt_name.mid(2);
          inverse = true;
@@ -688,7 +696,7 @@ KCmdLineArgsStatic::findOption(const KCmdLineOptions &options, QByteArray &opt,
                   nextOption = nextOption.left(p);
                if (nextOption.startsWith('!'))
                   nextOption = nextOption.mid(1);
-               if (nextOption.startsWith("no"))
+               if (nextOption.startsWith("no") && !nextOption.contains('<')) // krazy:exclude=strings
                {
                   nextOption = nextOption.mid(2);
                   enabled = !enabled;
@@ -727,10 +735,6 @@ KCmdLineArgsStatic::findOption(const QByteArray &optv, const QByteArray &_opt,
       argument = opt.mid(j+1);
       opt = opt.left(j);
    }
-#ifdef Q_WS_MACX
-   if(opt.startsWith("psn_"))
-      opt = "psn";
-#endif
 
    bool enabled = true;
    int result = 0;
@@ -742,14 +746,14 @@ KCmdLineArgsStatic::findOption(const QByteArray &optv, const QByteArray &_opt,
       ++args;
    }
    if ((args == s->argsList->end()) &&
-       (optv.startsWith('-') && !optv.startsWith("--")))
+       (optv.startsWith('-') && !optv.startsWith("--"))) // krazy:exclude=strings
    {
       // Option not found check if it is a valid option
       // in the style of -Pprinter1 or ps -aux
       int p = 1;
       while (true)
       {
-         QByteArray singleCharOption = " ";
+         QByteArray singleCharOption = " "; // krazy:exclude=doublequote_chars
          singleCharOption[0] = optv[p];
          args = s->argsList->begin();
          while (args != s->argsList->end())
@@ -791,10 +795,6 @@ KCmdLineArgsStatic::findOption(const QByteArray &optv, const QByteArray &_opt,
    {
       if (s->ignoreUnknown)
          return;
-#ifdef Q_WS_MACX
-      if (_opt.startsWith("psn_"))
-         return;
-#endif
       KCmdLineArgs::enable_i18n();
       KCmdLineArgs::usageError( i18n("Unknown option '%1'.", QString::fromLocal8Bit(_opt)));
    }
@@ -811,22 +811,18 @@ KCmdLineArgsStatic::findOption(const QByteArray &optv, const QByteArray &_opt,
       {
          if (s->ignoreUnknown)
             return;
-#ifdef Q_WS_MACX
-         if (_opt.startsWith("psn_"))
-		    return;
-#endif
          KCmdLineArgs::enable_i18n();
          KCmdLineArgs::usageError( i18n("Unknown option '%1'.", QString::fromLocal8Bit(_opt)));
       }
       if (argument.isEmpty())
       {
          i++;
-         if (i >= s->argc)
+         if (i >= s->all_argc)
          {
             KCmdLineArgs::enable_i18n();
             KCmdLineArgs::usageError( i18nc("@info:shell %1 is cmdoption name","'%1' missing.",  QString::fromLocal8Bit(opt_name)));
          }
-         argument = s->argv[i];
+         argument = s->all_argv[i];
       }
       (*args)->d->setOption(opt, argument);
    }
@@ -851,20 +847,19 @@ KCmdLineArgsStatic::parseAllArgs()
        allowArgs = allowArgs || name.startsWith('+') || everythingAfterArgIsArgs;
      }
    }
-   for(int i = 1; i < s->argc; i++)
+   for(int i = 1; i < s->all_argc; i++)
    {
-      if (!s->argv[i])
+      if (!s->all_argv[i])
          continue;
 
-      if ((s->argv[i][0] == '-') && s->argv[i][1] && inOptions)
+      if ((s->all_argv[i][0] == '-') && s->all_argv[i][1] && inOptions)
       {
          bool enabled = true;
-         QByteArray orig = s->argv[i];
+         QByteArray orig = s->all_argv[i];
          QByteArray option = orig.mid(1);
          if (option.startsWith('-'))
          {
             option = option.mid(1);
-            s->argv[i]++;
             if (option.isEmpty())
             {
                inOptions = false;
@@ -875,21 +870,34 @@ KCmdLineArgsStatic::parseAllArgs()
          {
             KCmdLineArgs::usage();
          }
-         else if (option.startsWith("help-"))
+         else if (option.startsWith("help-")) // krazy:exclude=strings
          {
             KCmdLineArgs::usage(option.mid(5));
          }
+#ifdef Q_WS_MAC
+         // skip the finder -psn_* hint
+         else if (option.startsWith("psn_")) // krazy:exclude=strings
+         {
+            continue;
+         }
+#endif
          else if ((option == "version") || (option == "v"))
          {
-            s->printQ( QString("Qt: %1\n").arg(qVersion()));
-            s->printQ( QString("KDE: %1\n").arg(KDE_VERSION_STRING));
-            s->printQ( QString("%1: %2\n"). arg(s->about->programName()).arg(s->about->version()));
+            KCmdLineArgs::enable_i18n();
+            s->printQ(i18nc("@info:shell message on appcmd --version; do not translate 'Development Platform'"
+                            "%3 application name, other %n version strings",
+                            "Qt: %1\n"
+                            "KDE Development Platform: %2\n"
+                            "%3: %4\n",
+                            QString::fromLatin1(qVersion()),
+                            QString::fromLatin1(KDE_VERSION_STRING),
+                            s->about->programName(), s->about->version()));
             exit(0);
          } else if (option == "license")
          {
             KCmdLineArgs::enable_i18n();
-            s->printQ( s->about->license() );
-            s->printQ( "\n" );
+            s->printQ(s->about->license());
+            s->printQ(QString::fromLatin1("\n"));
             exit(0);
          } else if (option == "author") {
              KCmdLineArgs::enable_i18n();
@@ -900,8 +908,8 @@ KCmdLineArgsStatic::parseAllArgs()
            for (QList<KAboutPerson>::ConstIterator it = authors.begin(); it != authors.end(); ++it ) {
              QString email;
              if ( !(*it).emailAddress().isEmpty() )
-               email = " &lt;" + (*it).emailAddress() + "&gt;";
-             authorlist += QString("    ") + (*it).name() + email + '\n';
+               email = QString::fromLatin1(" &lt;") + (*it).emailAddress() + QLatin1String("&gt;");
+             authorlist += QString::fromLatin1("    ") + (*it).name() + email + QLatin1Char('\n');
            }
            s->printQ( i18nc("the 2nd argument is a list of name+address, one on each line","%1 was written by\n%2",   QString(s->about->programName()) ,  authorlist ) );
          }
@@ -912,22 +920,34 @@ KCmdLineArgsStatic::parseAllArgs()
        {
          if (!s->about->customAuthorTextEnabled ())
          {
-           if (s->about->bugAddress().isEmpty() || s->about->bugAddress() == "submit@bugs.kde.org" )
+           if (s->about->bugAddress().isEmpty() || s->about->bugAddress() == QLatin1String("submit@bugs.kde.org") )
              s->printQ( i18n( "Please use http://bugs.kde.org to report bugs.\n" ) );
            else
              s->printQ( i18n( "Please report bugs to %1.\n" , s->about->bugAddress()) );
          }
          else
          {
-           s->printQ(s->about->customAuthorPlainText());
+           s->printQ(s->about->customAuthorPlainText()+QLatin1Char('\n'));
          }
        }
        exit(0);
          } else {
-           if (option.startsWith("no"))
+           if (option.startsWith("no")) // krazy:exclude=strings
            {
-              option = option.mid(2);
-              enabled = false;
+              bool noHasParameter=false;
+              foreach(const QByteArray& name, appOptions->d->options.d->names)
+              {
+                 if (name.contains(option + QByteArray(" ")) && name.contains('<'))
+                 {
+                    noHasParameter=true;
+                    break;
+                  }
+              }
+              if (!noHasParameter)
+              {
+                 option = option.mid(2);
+                 enabled = false;
+              }
            }
            s->findOption(orig, option, i, enabled, inOptions);
          }
@@ -940,11 +960,11 @@ KCmdLineArgsStatic::parseAllArgs()
             if (s->ignoreUnknown)
                continue;
             KCmdLineArgs::enable_i18n();
-            KCmdLineArgs::usageError(i18n("Unexpected argument '%1'.", KuitSemantics::escape(s->decodeInput(s->argv[i]))));
+            KCmdLineArgs::usageError(i18n("Unexpected argument '%1'.", KuitSemantics::escape(s->decodeInput(s->all_argv[i]))));
          }
          else
          {
-            appOptions->d->addArgument(s->argv[i]);
+            appOptions->d->addArgument(s->all_argv[i]);
             if (everythingAfterArgIsArgs)
                 inOptions = false;
          }
@@ -970,7 +990,7 @@ int & KCmdLineArgs::qtArgc()
 
    KCmdLineArgs *args = parsedArgs("qt");
    Q_ASSERT(args); // No qt options have been added!
-   if (!s->argv)
+   if (!s->all_argv)
    {
       fprintf(stderr, "\n\nFAILURE (KCmdLineArgs):\n");
       fprintf(stderr, "Application has not called KCmdLineArgs::init(...).\n\n");
@@ -979,7 +999,7 @@ int & KCmdLineArgs::qtArgc()
       exit(255);
    }
 
-   Q_ASSERT(s->argc >= (args->count()+1));
+   Q_ASSERT(s->all_argc >= (args->count()+1));
    qt_argc = args->count() +1;
    return qt_argc;
 }
@@ -998,15 +1018,22 @@ KCmdLineArgs::qtArgv()
    if (!(s->mStdargs & KCmdLineArgs::CmdLineArgQt))
    {
      s_qt_argv = new char*[2];
-     s_qt_argv[0] = qstrdup(s->argc?s->argv[0]:"");
+     s_qt_argv[0] = qstrdup(s->all_argc?s->all_argv[0]:"");
      s_qt_argv[1] = 0;
 
      return s_qt_argv;
    }
 
    KCmdLineArgs *args = parsedArgs("qt");
-   Q_ASSERT(args); // No qt options have been added!
-   if (!s->argv)
+   if (!args)
+   {
+      fprintf(stderr, "\n\nFAILURE (KCmdLineArgs):\n");
+      fprintf(stderr, "The \"qt\" options have not be added to KCmdLineArgs!\n\n");
+
+      assert( 0 );
+      exit(255);
+   }
+   if (!s->all_argv)
    {
       fprintf(stderr, "\n\nFAILURE (KCmdLineArgs):\n");
       fprintf(stderr, "Application has not called KCmdLineArgs::init(...).\n\n");
@@ -1017,7 +1044,7 @@ KCmdLineArgs::qtArgv()
 
    int count=args->count();
    s_qt_argv = new char*[ count + 2 ];
-   s_qt_argv[0] = qstrdup(s->argc?s->argv[0]:"");
+   s_qt_argv[0] = qstrdup(s->all_argc?s->all_argv[0]:"");
    int i = 0;
    for(; i < count; i++)
    {
@@ -1055,11 +1082,11 @@ KCmdLineArgs::usageError(const QString &error)
     QByteArray localError = s->encodeOutput(error);
     if (localError.endsWith('\n'))
         localError.chop(1);
-    fprintf(stderr, "%s: %s\n", s->argv[0], localError.data());
+    fprintf(stderr, "%s: %s\n", s->appName, localError.data());
 
     QString tmp = i18n("Use --help to get a list of available command line options.");
     localError = s->encodeOutput(tmp);
-    fprintf(stderr, "%s: %s\n", s->argv[0], localError.data());
+    fprintf(stderr, "%s: %s\n", s->appName, localError.data());
     exit(254);
 }
 
@@ -1070,8 +1097,8 @@ KCmdLineArgs::usage(const QByteArray &id)
    Q_ASSERT(s->argsList != 0); // It's an error to call usage(...) without
                                // having done addCmdLineOptions first!
 
-   QString optionFormatString   = "  %1 %2\n";
-   QString optionFormatStringDef  = "  %1 %2 [%3]\n";
+   QString optionFormatString = QString::fromLatin1("  %1 %2\n");
+   QString optionFormatStringDef = QString::fromLatin1("  %1 %2 [%3]\n");
    QString tmp;
    QString usage;
 
@@ -1087,7 +1114,7 @@ KCmdLineArgs::usage(const QByteArray &id)
    {
       if (!(*args)->d->name.isEmpty())
       {
-         usage = i18n("[%1-options]", (*args)->d->name.toString())+' '+usage;
+         usage = i18n("[%1-options]", (*args)->d->name.toString())+QLatin1Char(' ')+usage;
       }
       if (args == s->argsList->begin())
          break;
@@ -1102,24 +1129,25 @@ KCmdLineArgs::usage(const QByteArray &id)
      {
        QByteArray opt_name = option.d->names[i];
        if (opt_name.startsWith('+'))
-          usage = usage + (opt_name.mid(1)) + ' ';
+           usage += QString::fromLatin1(opt_name.mid(1)) + QLatin1Char(' ');
        else if ( opt_name.startsWith("!+") )
-          usage = usage + (opt_name.mid(2)) + ' ';
+          usage += QString::fromLatin1(opt_name.mid(2)) + QLatin1Char(' ');
      }
    }
 
-   s->printQ(i18n("Usage: %1 %2\n", s->argv[0], KuitSemantics::escape(usage)));
-   s->printQ('\n'+s->about->shortDescription()+'\n');
+   s->printQ(i18n("Usage: %1 %2\n", QString::fromLocal8Bit(s->appName), KuitSemantics::escape(usage)));
+   s->printQ(QLatin1Char('\n')+s->about->shortDescription()+QLatin1Char('\n'));
 
    s->printQ(i18n("\nGeneric options:\n"));
-   s->printQ(optionFormatString.arg("--help", -25).arg(i18n("Show help about options")));
+   s->printQ(optionFormatString.arg(QString::fromLatin1("--help"), -25)
+             .arg(i18n("Show help about options")));
 
    args = s->argsList->begin();
    while(args != s->argsList->end())
    {
       if (!(*args)->d->name.isEmpty() && !(*args)->d->id.isEmpty())
       {
-         QString option = QString("--help-%1").arg(QString::fromLatin1((*args)->d->id));
+          QString option = QString::fromLatin1("--help-%1").arg(QString::fromLatin1((*args)->d->id));
          QString desc = i18n("Show %1 specific options", (*args)->d->name.toString());
 
          s->printQ(optionFormatString.arg(option, -25).arg(desc));
@@ -1127,11 +1155,11 @@ KCmdLineArgs::usage(const QByteArray &id)
       ++args;
    }
 
-   s->printQ(optionFormatString.arg("--help-all",-25).arg(i18n("Show all options")));
-   s->printQ(optionFormatString.arg("--author",-25).arg(i18n("Show author information")));
-   s->printQ(optionFormatString.arg("-v, --version",-25).arg(i18n("Show version information")));
-   s->printQ(optionFormatString.arg("--license",-25).arg(i18n("Show license information")));
-   s->printQ(optionFormatString.arg("--", -25).arg(i18n("End of options")));
+   s->printQ(optionFormatString.arg(QString::fromLatin1("--help-all"),-25).arg(i18n("Show all options")));
+   s->printQ(optionFormatString.arg(QString::fromLatin1("--author"),-25).arg(i18n("Show author information")));
+   s->printQ(optionFormatString.arg(QString::fromLatin1("-v, --version"),-25).arg(i18n("Show version information")));
+   s->printQ(optionFormatString.arg(QString::fromLatin1("--license"),-25).arg(i18n("Show license information")));
+   s->printQ(optionFormatString.arg(QString::fromLatin1("--"), -25).arg(i18n("End of options")));
 
    args = s->argsList->begin(); // Sets current to 1st.
 
@@ -1176,9 +1204,9 @@ KCmdLineArgs::usage(const QByteArray &id)
          {
             if (!descriptionFull.isEmpty())
             {
-               optionsHeader = '\n'+descriptionFull;
-               if (!optionsHeader.endsWith('\n'))
-                  optionsHeader.append('\n');
+               optionsHeader = QLatin1Char('\n')+descriptionFull;
+               if (!optionsHeader.endsWith(QLatin1Char('\n')))
+                  optionsHeader.append(QLatin1Char('\n'));
                hasOptions = false;
             }
             continue;
@@ -1189,9 +1217,9 @@ KCmdLineArgs::usage(const QByteArray &id)
          {
             if (!descriptionFull.isEmpty())
             {
-               tmp = '\n'+descriptionFull;
-               if (!tmp.endsWith('\n'))
-                  tmp.append('\n');
+               tmp = QLatin1Char('\n')+descriptionFull;
+               if (!tmp.endsWith(QLatin1Char('\n')))
+                  tmp.append(QLatin1Char('\n'));
                s->printQ(tmp);
             }
             continue;
@@ -1200,7 +1228,7 @@ KCmdLineArgs::usage(const QByteArray &id)
          // Options
          if (!descriptionFull.isEmpty())
          {
-            dl = descriptionFull.split( '\n', QString::KeepEmptyParts);
+            dl = descriptionFull.split(QLatin1Char('\n'), QString::KeepEmptyParts);
             description = dl.first();
             dl.erase( dl.begin() );
          }
@@ -1242,11 +1270,11 @@ KCmdLineArgs::usage(const QByteArray &id)
                opt = opt + name;
                if (option.d->defaults[i].isEmpty())
                {
-                  s->printQ(optionFormatString.arg(QString( opt ), -25).arg(description));
+                   s->printQ(optionFormatString.arg(QString::fromLatin1(opt), -25).arg(description));
                }
                else
                {
-                  s->printQ(optionFormatStringDef.arg(QString( opt ), -25)
+                   s->printQ(optionFormatStringDef.arg(QString::fromLatin1(opt), -25)
                             .arg(description, option.d->defaults[i]));
                }
                opt.clear();
@@ -1256,7 +1284,7 @@ KCmdLineArgs::usage(const QByteArray &id)
              it != dl.end();
              ++it)
          {
-            s->printQ(optionFormatString.arg("", -25).arg(*it));
+            s->printQ(optionFormatString.arg(QString(), -25).arg(*it));
          }
        }
 
@@ -1312,9 +1340,7 @@ KCmdLineArgs::clear()
 void
 KCmdLineArgs::reset()
 {
-   if ( s->argsList ) {
-      delete s->argsList; s->argsList = 0;
-   }
+   delete s->argsList; s->argsList = 0;
    s->parsed = false;
 }
 
@@ -1357,7 +1383,7 @@ KCmdLineArgsPrivate::setOption(const QByteArray &opt, bool enabled)
    if (isQt)
    {
       // Qt does it own parsing.
-      QByteArray argString = "-";
+      QByteArray argString = "-"; // krazy:exclude=doublequote_chars
       if( !enabled )
           argString += "no";
       argString += opt;
@@ -1368,9 +1394,9 @@ KCmdLineArgsPrivate::setOption(const QByteArray &opt, bool enabled)
    }
 
    if (enabled)
-      parsedOptionList->insert( opt, "t" );
+      parsedOptionList->insert( opt, "t" ); // krazy:exclude=doublequote_chars
    else
-      parsedOptionList->insert( opt, "f" );
+      parsedOptionList->insert( opt, "f" ); // krazy:exclude=doublequote_chars
 }
 
 void
@@ -1379,7 +1405,7 @@ KCmdLineArgsPrivate::setOption(const QByteArray &opt, const QByteArray &value)
    if (isQt)
    {
       // Qt does it's own parsing.
-      QByteArray argString = "-";
+      QByteArray argString = "-"; // krazy:exclude=doublequote_chars
       argString += opt;
       addArgument(argString);
       addArgument(value);
@@ -1547,7 +1573,7 @@ KUrl KCmdLineArgs::makeURL(const QByteArray &_urlArg)
 
     if ( KUrl::isRelativeUrl(urlArg) || fileInfo.exists() ) {
         KUrl result;
-        result.setPath( cwd()+'/'+urlArg );
+        result.setPath(cwd()+QLatin1Char('/')+urlArg);
         result.cleanPath();
         return result;  // Relative path
     }
@@ -1577,3 +1603,16 @@ bool KCmdLineArgs::isTempFileSet()
     KCmdLineArgs* args = KCmdLineArgs::parsedArgs( "kde-tempfile" );
     return args && args->isSet( "tempfile" );
 }
+
+QStringList KCmdLineArgs::allArguments()
+{
+    QStringList lst;
+
+    for(int i = 0; i < s->all_argc; i++) {
+        char* arg = s->all_argv[i];
+        if (!arg)
+            continue;
+        lst.append(QString::fromLocal8Bit(arg));
+    }
+    return lst;
+}
