@@ -55,7 +55,7 @@
  * If the existing file is outdated, it will not get read
  * but instead we'll ask kded to regenerate a new one...
  */
-#define KSYCOCA_VERSION 162
+#define KSYCOCA_VERSION 206
 
 /**
  * Sycoca file name, used internally (by kbuildsycoca)
@@ -108,11 +108,11 @@ KSycocaPrivate::KSycocaPrivate()
 }
 
 void KSycocaPrivate::setStrategyFromString(const QString& strategy) {
-    if (strategy == "mmap")
+    if (strategy == QLatin1String("mmap"))
         m_sycocaStrategy = StrategyMmap;
-    else if (strategy == "file")
+    else if (strategy == QLatin1String("file"))
         m_sycocaStrategy = StrategyFile;
-    else if (strategy == "sharedmem")
+    else if (strategy == QLatin1String("sharedmem"))
         m_sycocaStrategy = StrategyMemFile;
     else if (!strategy.isEmpty())
         kWarning(7011) << "Unknown sycoca strategy:" << strategy;
@@ -181,7 +181,8 @@ KSycoca::KSycoca()
   : d(new KSycocaPrivate)
 {
     QDBusConnection::sessionBus().connect(QString(), QString(),
-                                          "org.kde.KSycoca", "notifyDatabaseChanged",
+                                          QString::fromLatin1("org.kde.KSycoca"),
+                                          QString::fromLatin1("notifyDatabaseChanged"),
                                           this, SLOT(notifyDatabaseChanged(QStringList)));
 }
 
@@ -193,7 +194,7 @@ bool KSycocaPrivate::openDatabase(bool openDummyIfNotFound)
     QString path = KSycoca::absoluteFilePath();
 
     bool canRead = KDE::access(path, R_OK) == 0;
-    kDebug(7011) << "Trying to open ksycoca from " << path;
+    kDebug(7011) << "Trying to open ksycoca from" << path;
     if (!canRead) {
         path = KSycoca::absoluteFilePath(KSycoca::GlobalDatabase);
         if (!path.isEmpty()) {
@@ -246,13 +247,14 @@ KSycocaAbstractDevice* KSycocaPrivate::device()
             }
         }
 #endif
+#ifndef QT_NO_SHAREDMEMORY
         if (!device && m_sycocaStrategy == StrategyMemFile) {
             device = new KSycocaMemFileDevice(m_databasePath);
             if (!device->device()->open(QIODevice::ReadOnly)) {
                 delete device; device = 0;
             }
         }
-
+#endif
         if (!device) {
             device = new KSycocaFileDevice(m_databasePath);
             if (!device->device()->open(QIODevice::ReadOnly)) {
@@ -341,10 +343,12 @@ void KSycoca::addFactory( KSycocaFactory *factory )
     d->addFactory(factory);
 }
 
+#ifndef KDE_NO_DEPRECATED
 bool KSycoca::isChanged(const char *type)
 {
-    return self()->d->changeList.contains(type);
+    return self()->d->changeList.contains(QString::fromLatin1(type));
 }
+#endif
 
 void KSycoca::notifyDatabaseChanged(const QStringList &changeList)
 {
@@ -357,7 +361,9 @@ void KSycoca::notifyDatabaseChanged(const QStringList &changeList)
     d->closeDatabase();
 
     // Now notify applications
+#ifndef KDE_NO_DEPRECATED
     emit databaseChanged();
+#endif
     emit databaseChanged(changeList);
 }
 
@@ -410,7 +416,7 @@ bool KSycocaPrivate::checkDatabase(BehaviorsIfNotFound ifNotFound)
 
     // We can only use the installed ksycoca file if kdeinit+klauncher+kded are running,
     // since kded is what keeps the file uptodate.
-    const bool kdeinitRunning = QDBusConnection::sessionBus().interface()->isServiceRegistered("org.kde.klauncher");
+    const bool kdeinitRunning = QDBusConnection::sessionBus().interface()->isServiceRegistered(QString::fromLatin1("org.kde.klauncher"));
 
     // Check if new database already available
     if (kdeinitRunning && openDatabase(ifNotFound & IfNotFoundOpenDummy)) {
@@ -429,7 +435,7 @@ bool KSycocaPrivate::checkDatabase(BehaviorsIfNotFound ifNotFound)
             // and since kdeinit4 only returns after kbuildsycoca4 is done, we can proceed.
         } else {
             kDebug(7011) << QThread::currentThread() << "We have no database.... launching" << KBUILDSYCOCA_EXENAME;
-            if (QProcess::execute(KStandardDirs::findExe(KBUILDSYCOCA_EXENAME)) != 0)
+            if (QProcess::execute(KStandardDirs::findExe(QString::fromLatin1(KBUILDSYCOCA_EXENAME))) != 0)
                 qWarning("ERROR: Running KSycoca failed.");
         }
 
@@ -522,14 +528,18 @@ quint32 KSycoca::updateSignature()
 
 QString KSycoca::absoluteFilePath(DatabaseType type)
 {
-   if (type == GlobalDatabase)
-      return KStandardDirs::locate("services", KSYCOCA_FILENAME);
+    if (type == GlobalDatabase) {
+        QString path = KGlobal::dirs()->findResource("services", QString::fromLatin1(KSYCOCA_FILENAME));
+        if (path.isEmpty())
+            return KGlobal::dirs()->saveLocation("services") + QString::fromLatin1(KSYCOCA_FILENAME);
+        return path;
+    }
 
-   const QByteArray ksycoca_env = qgetenv("KDESYCOCA");
-   if (ksycoca_env.isEmpty())
-      return KGlobal::dirs()->saveLocation("cache") + KSYCOCA_FILENAME;
-   else
-      return QFile::decodeName(ksycoca_env);
+    const QByteArray ksycoca_env = qgetenv("KDESYCOCA");
+    if (ksycoca_env.isEmpty())
+        return KGlobal::dirs()->saveLocation("cache") + QString::fromLatin1(KSYCOCA_FILENAME);
+    else
+        return QFile::decodeName(ksycoca_env);
 }
 
 QString KSycoca::language()
@@ -555,7 +565,7 @@ void KSycoca::flagError()
     d->readError = true;
     if (s_autoRebuild) {
         // Rebuild the damned thing.
-        if (QProcess::execute(KStandardDirs::findExe(KBUILDSYCOCA_EXENAME)) != 0)
+        if (QProcess::execute(KStandardDirs::findExe(QString::fromLatin1(KBUILDSYCOCA_EXENAME))) != 0)
             qWarning("ERROR: Running %s failed", KBUILDSYCOCA_EXENAME);
         // Old comment, maybe not true anymore:
         // Do not wait until the DBUS signal from kbuildsycoca here.
@@ -563,10 +573,12 @@ void KSycoca::flagError()
     }
 }
 
+#ifndef KDE_NO_DEPRECATED
 bool KSycoca::readError() // KDE5: remove
 {
     return false;
 }
+#endif
 
 bool KSycoca::isBuilding()
 {
