@@ -43,16 +43,18 @@
 
 #define POOL_USAGE 0
 
+Q_DECLARE_METATYPE(QModelIndex);
+
 /**
   Private class that helps to provide binary compatibility between releases.
   @internal
 */
 //@cond PRIVATE
-class EventListener
+class KWidgetItemDelegateEventListener
     : public QObject
 {
 public:
-    EventListener(KWidgetItemDelegatePoolPrivate *poolPrivate, QObject *parent = 0)
+    KWidgetItemDelegateEventListener(KWidgetItemDelegatePoolPrivate *poolPrivate, QObject *parent = 0)
         : QObject(parent)
         , poolPrivate(poolPrivate)
     {
@@ -66,7 +68,7 @@ private:
 
 KWidgetItemDelegatePoolPrivate::KWidgetItemDelegatePoolPrivate(KWidgetItemDelegate *d)
     : delegate(d)
-    , eventListener(new EventListener(this))
+    , eventListener(new KWidgetItemDelegateEventListener(this))
     , clearing(false)
 {
 }
@@ -106,7 +108,10 @@ QList<QWidget*> KWidgetItemDelegatePool::findWidgets(const QPersistentModelIndex
     if (d->usedWidgets.contains(index)) {
         result = d->usedWidgets[index];
     } else {
+        // ### KDE5 This sets a property on the delegate because we can't add an argument to createItemWidgets
+        d->delegate->setProperty("goya:creatingWidgetForIndex", QVariant::fromValue(index));
         result = d->delegate->createItemWidgets();
+        d->delegate->setProperty("goya:creatingWidgetForIndex", QVariant());
         d->allocatedWidgets << result;
         d->usedWidgets[index] = result;
         foreach (QWidget *widget, result) {
@@ -160,12 +165,12 @@ void KWidgetItemDelegatePool::fullClear()
     d->widgetInIndex.clear();
 }
 
-bool EventListener::eventFilter(QObject *watched, QEvent *event)
+bool KWidgetItemDelegateEventListener::eventFilter(QObject *watched, QEvent *event)
 {
     QWidget *widget = static_cast<QWidget*>(watched);
 
     if (event->type() == QEvent::Destroy && !poolPrivate->clearing) {
-        kWarning() << "User of KWidgetItemDelegate should not delete widgets created by createWidgets!";
+        kWarning() << "User of KWidgetItemDelegate should not delete widgets created by createItemWidgets!";
         // assume the application has kept a list of widgets and tries to delete them manually
         // they have been reparented to the view in any case, so no leaking occurs
         poolPrivate->widgetInIndex.remove(widget);
