@@ -62,12 +62,12 @@ private Q_SLOTS:
         w.removeTab(0);
         for (int i = 0; i < 5; ++i) {
             //kDebug() << i << w.tabText(i);
-            QCOMPARE(w.tabText(i), prefix+QString::number(i+1));
+            QCOMPARE(w.tabText(i), QString(prefix+QString::number(i+1)));
         }
         w.removeTab(0);
         for (int i = 0; i < 4; ++i) {
             //kDebug() << i << w.tabText(i);
-            QCOMPARE(w.tabText(i), prefix+QString::number(i+2));
+            QCOMPARE(w.tabText(i), QString(prefix+QString::number(i+2)));
         }
     }
     void testMoveTab()
@@ -108,10 +108,10 @@ private Q_SLOTS:
         w->moveTab(0,3);
         //for (int i = 0; i < 4; ++i)
             //kDebug() << i << w->tabText(i);
-        QCOMPARE(w->tabText(0), prefix+QString::number(1));
-        QCOMPARE(w->tabText(1), prefix+QString::number(2));
-        QCOMPARE(w->tabText(2), prefix+QString::number(3));
-        QCOMPARE(w->tabText(3), prefix+QString::number(0));
+        QCOMPARE(w->tabText(0), QString(prefix+QString::number(1)));
+        QCOMPARE(w->tabText(1), QString(prefix+QString::number(2)));
+        QCOMPARE(w->tabText(2), QString(prefix+QString::number(3)));
+        QCOMPARE(w->tabText(3), QString(prefix+QString::number(0)));
 
         // Did the focus switch to the lineEdit, due to removeTab+insertTab? Whoops.
         QCOMPARE(topLevel.focusWidget()->objectName(), lineEdits[0]->objectName());
@@ -120,7 +120,7 @@ private Q_SLOTS:
         QCOMPARE(topLevel.focusWidget()->objectName(), lineEdits[0]->objectName());
         for (int i = 0; i < 4; ++i) {
             //kDebug() << i << w->tabText(i);
-            QCOMPARE(w->tabText(i), prefix+QString::number(i));
+            QCOMPARE(w->tabText(i), QString(prefix+QString::number(i)));
         }
      }
 
@@ -136,6 +136,7 @@ private Q_SLOTS:
      }
 
      void testMiddleClickTabReordering();
+     void testTabMoved();
 
 private Q_SLOTS:
     void slotCurrentChanged(int index)
@@ -161,7 +162,7 @@ public:
 void KTabWidget_UnitTest::testMiddleClickTabReordering()
 {
     MyTabWidget tabWidget;
-    tabWidget.setTabReorderingEnabled(true);
+    tabWidget.setMovable(true);
 
     QWidget* w0 = new QWidget;
     QWidget* w1 = new QWidget;
@@ -193,6 +194,25 @@ void KTabWidget_UnitTest::testMiddleClickTabReordering()
     QTest::mouseRelease(tabWidget.getTabBar(), Qt::MidButton, Qt::NoModifier, pos1);
 }
 
+void KTabWidget_UnitTest::testTabMoved()
+{
+    MyTabWidget tabWidget;
+    tabWidget.setMovable(true);
+    tabWidget.setAutomaticResizeTabs(true);
+
+    tabWidget.insertTab(0, new QWidget, "Tab A");
+    tabWidget.insertTab(1, new QWidget, "Tab B");
+    tabWidget.insertTab(2, new QWidget, "Tab C");
+    tabWidget.insertTab(3, new QWidget, "Tab D");
+
+    tabWidget.getTabBar()->moveTab(3, 1);
+
+    QCOMPARE(tabWidget.tabText(0), QString("Tab A"));
+    QCOMPARE(tabWidget.tabText(1), QString("Tab D"));
+    QCOMPARE(tabWidget.tabText(2), QString("Tab B"));
+    QCOMPARE(tabWidget.tabText(3), QString("Tab C"));
+}
+
 QTEST_KDEMAIN(KTabWidget_UnitTest, GUI)
 
 #include "ktabwidget_unittest.moc"
