@@ -864,11 +864,6 @@ void KToolBar::addXMLGUIClient( KXMLGUIClient *client )
     d->xmlguiClients << client;
 }
 
-void KToolBar::removeXMLGUIClient( KXMLGUIClient *client )
-{
-    d->xmlguiClients.remove(client);
-}
-
 void KToolBar::contextMenuEvent(QContextMenuEvent* event)
 {
 #ifndef KDE_NO_DEPRECATED
