@@ -34,6 +34,7 @@
 #include "wtf/PassRefPtr.h"
 #include "misc/htmlnames.h"
 #include "dom/QualifiedName.h"
+#include "xml/dom2_eventsimpl.h"
 
 template <class type> class QList;
 class KHTMLView;
@@ -58,40 +59,6 @@ class RegisteredEventListener;
 class EventImpl;
 class Selection;
 
-struct RegisteredListenerList {
-    RegisteredListenerList() : listeners(0)
-    {}
-
-    ~RegisteredListenerList();
-
-    void addEventListener(EventName id, EventListener *listener, const bool useCapture);
-    void removeEventListener(EventName id, EventListener *listener, bool useCapture);
-
-    void setHTMLEventListener(EventName id, EventListener *listener);
-    EventListener *getHTMLEventListener(EventName id);
-
-    bool hasEventListener(EventName id);
-    void clear();
-
-    //### KDE4: should disappear
-    bool stillContainsListener(const RegisteredEventListener& listener);
-
-    QList<RegisteredEventListener>* listeners;//The actual listener list - may be 0
-private:
-    bool isHTMLEventListener(EventListener* listener);
-};
-
-class EventTargetImpl : public khtml::TreeShared<EventTargetImpl>
-{
-public:
-    enum Type {
-        DOM_NODE,
-        XML_HTTP_REQUEST
-    };
-
-    virtual Type eventTargetType() const = 0;
-};
-
 class NodeImpl : public EventTargetImpl
 {
     friend class DocumentImpl;
@@ -131,6 +98,10 @@ public:
     // DOM3. See the wrapper (DOM::Node for the constants used in the return value
     unsigned compareDocumentPosition(const DOM::NodeImpl* other);
 
+    // WA Selector API L1. It's specified only for some types, but we provide it here
+    WTF::PassRefPtr<DOM::ElementImpl>  querySelector(const DOM::DOMString& query, int& ec);
+    WTF::PassRefPtr<DOM::NodeListImpl> querySelectorAll(const DOM::DOMString& query, int& ec);
+
     // insertBefore, replaceChild and appendChild also close newChild
     // unlike the speed optimized addChild (which is used by the parser)
     virtual NodeImpl *insertBefore ( NodeImpl *newChild, NodeImpl *refChild, int &exceptioncode );
@@ -294,9 +265,19 @@ public:
  
     virtual short tabIndex() const { return 0; }
 
-    virtual bool isFocusable() const { return false; }
-    virtual bool isMouseFocusable() const { return isFocusable(); }
-    virtual bool isTabFocusable() const { return isFocusable(); }
+    enum FocusType {
+        FT_Any,
+        FT_Mouse,
+        FT_Tab
+    };
+
+    // Elements that are focusable by default should override this.
+    // Warning: if they're in a language that supports tabIndex (e.g. HTML),
+    // they must call back to the base class whenever hasTabIndex() is set.
+    virtual bool isFocusableImpl(FocusType) const { return false; }
+    bool isFocusable() const { return isFocusableImpl(FT_Any); }
+    bool isMouseFocusable() const { return isFocusableImpl(FT_Mouse); }
+    bool isTabFocusable() const { return isFocusableImpl(FT_Tab); }
 
     virtual bool isInline() const;
 
@@ -317,18 +298,23 @@ public:
     DocumentImpl* document() const { return m_document.get(); }
     void setDocument(DocumentImpl* doc);
 
-    void addEventListener(EventName id, EventListener *listener, const bool useCapture);
-    void removeEventListener(EventName id, EventListener *listener, bool useCapture);
-    void setHTMLEventListener(EventName id, EventListener *listener);
-    void setHTMLEventListener(unsigned id, EventListener *listener);
-    EventListener *getHTMLEventListener(EventName id);
-    EventListener *getHTMLEventListener(unsigned id);
+    virtual DocumentImpl* eventTargetDocument();
 
     void dispatchEvent(EventImpl *evt, int &exceptioncode, bool tempEvent = false);
+
+    // takes care of bubbling and the like. The target is generally 'this',
+    // unless the event specifies something special like Window as the target,
+    // in which case that's used for dispatch.
     void dispatchGenericEvent( EventImpl *evt, int &exceptioncode);
+    
     // return true if event not prevented
     bool dispatchHTMLEvent(int _id, bool canBubbleArg, bool cancelableArg);
+
+    // Window events are special in that they're only dispatched on Window, and not
+    // the current node.
     void dispatchWindowEvent(int _id, bool canBubbleArg, bool cancelableArg);
+    void dispatchWindowEvent(EventImpl* evt);
+    
     void dispatchMouseEvent(QMouseEvent *e, int overrideId = 0, int overrideDetail = 0);
     void dispatchUIEvent(int _id, int detail = 0);
     void dispatchSubtreeModifiedEvent();
@@ -336,13 +322,6 @@ public:
     // this matches the logic in KHTMLView.
     bool dispatchKeyEvent(QKeyEvent *key, bool keypress);
 
-    void handleLocalEvents(EventImpl *evt, bool useCapture);
-
-    /**
-     * Perform the default action for an event e.g. submitting a form
-     */
-    virtual void defaultEventHandler(EventImpl *evt);
-
     virtual bool isReadOnly();
     virtual bool childTypeAllowed( unsigned short /*type*/ ) { return false; }
     virtual unsigned long childNodeCount();
@@ -509,16 +488,19 @@ public:
 
     // FOR SVG Events support (WebCore API compatibility)
     QList<RegisteredEventListener>* localEventListeners() {
-        return m_regdListeners.listeners;
+        return listenerList().listeners;
     }
 
+    DOMString lookupNamespaceURI( const DOMString& prefix );
+
 private: // members
     khtml::DocPtr<DocumentImpl> m_document;
     NodeImpl *m_previous;
     NodeImpl *m_next;
+
+    NodeImpl* findNextElementAncestor( NodeImpl* node );
 protected:
     khtml::RenderObject *m_render;
-    RegisteredListenerList m_regdListeners;
 
     bool m_hasId : 1;
     bool m_attached : 1;
