@@ -15,17 +15,19 @@
 
 #include <kio/kio_export.h>
 
-#include <kbuttongroup.h>
 #include <kcombobox.h>
 #include <klistwidgetsearchline.h>
-#include <kapplication.h>
 #include <klocale.h>
 #include <kstandarddirs.h>
 #include <kiconloader.h>
 #include <kfiledialog.h>
 #include <kimagefilepreview.h>
+#ifndef _WIN32_WCE
 #include <ksvgrenderer.h>
+#endif
 
+#include <QtGui/QApplication>
+#include <QtGui/QGroupBox>
 #include <QtGui/QLayout>
 #include <QtGui/QLabel>
 #include <QtCore/QTimer>
@@ -136,8 +138,8 @@ KIconCanvas::KIconCanvas(QWidget *parent)
     setIconSize(QSize(60, 60));
     d->mpTimer = new QTimer(this);
     connect(d->mpTimer, SIGNAL(timeout()), this, SLOT(_k_slotLoadFiles()));
-    connect(this, SIGNAL( currentItemChanged(QListWidgetItem *, QListWidgetItem *)),
-            this, SLOT(_k_slotCurrentChanged(QListWidgetItem *)));
+    connect(this, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)),
+            this, SLOT(_k_slotCurrentChanged(QListWidgetItem*)));
     setGridSize(QSize(100,80));
 
     d->mpDelegate = new KIconCanvasDelegate(this, itemDelegate());
@@ -202,14 +204,16 @@ void KIconCanvas::KIconCanvasPrivate::_k_slotLoadFiles()
 	if (ext != "SVG" && ext != "VGZ")
 	    img.load(*it);
 	else {
+#ifndef _WIN32_WCE
             // Special stuff for SVG icons
             img = QImage(canvasIconWidth, canvasIconHeight, QImage::Format_ARGB32_Premultiplied);
             img.fill(0);
-            KSvgRenderer renderer(*it);
+            QSvgRenderer renderer(*it);
             if (renderer.isValid()) {
                 QPainter p(&img);
                 renderer.render(&p);
             }
+#endif
         }
 
 	if (img.isNull())
@@ -405,7 +409,7 @@ void KIconDialog::KIconDialogPrivate::init()
 
 
     mpCanvas = new KIconCanvas(main);
-    connect(mpCanvas, SIGNAL(itemActivated(QListWidgetItem *)), q, SLOT(_k_slotAcceptIcons()));
+    connect(mpCanvas, SIGNAL(itemActivated(QListWidgetItem*)), q, SLOT(_k_slotAcceptIcons()));
     top->addWidget(mpCanvas);
     searchLine->setListWidget(mpCanvas);
 
@@ -633,7 +637,7 @@ QString KIconDialog::openDialog()
             return name;
         }
         QFileInfo fi(name);
-        return fi.baseName();
+        return fi.completeBaseName();
     }
     return QString();
 }
@@ -864,7 +868,7 @@ void KIconButton::setIcon(const QString& icon)
 
     if (!d->mpDialog) {
         d->mpDialog = new KIconDialog(d->mpLoader, this);
-        connect(d->mpDialog, SIGNAL(newIconName(const QString&)), this, SLOT(_k_newIconName(const QString&)));
+        connect(d->mpDialog, SIGNAL(newIconName(QString)), this, SLOT(_k_newIconName(QString)));
     }
 
     if (d->mbUser) {
@@ -893,7 +897,7 @@ void KIconButton::KIconButtonPrivate::_k_slotChangeIcon()
     if (!mpDialog)
     {
         mpDialog = new KIconDialog(mpLoader, q);
-        connect(mpDialog, SIGNAL(newIconName(const QString&)), q, SLOT(_k_newIconName(const QString&)));
+        connect(mpDialog, SIGNAL(newIconName(QString)), q, SLOT(_k_newIconName(QString)));
     }
 
     mpDialog->setup(mGroup, mContext, m_bStrictIconSize, iconSize, mbUser);
