@@ -1,6 +1,6 @@
 /* This file is part of the KDE libraries
     Copyright 2000       Stephan Kulow <coolo@kde.org>
-    Copyright 2000-2006  David Faure <faure@kde.org>
+    Copyright 2000-2009  David Faure <faure@kde.org>
     Copyright 2000       Waldo Bastian <bastian@kde.org>
 
     This library is free software; you can redistribute it and/or
@@ -22,23 +22,16 @@
 #include "deletejob.h"
 
 #include "kdirlister.h"
-#include "kmimetype.h"
 #include "scheduler.h"
 #include "kdirwatch.h"
 #include "kprotocolmanager.h"
 #include "jobuidelegate.h"
 #include <kdirnotify.h>
-#include <kuiserverjobtracker.h>
 
-#include <kauthorized.h>
 #include <klocale.h>
 #include <kdebug.h>
 #include <kde_file.h>
 
-#include <assert.h>
-#include <stdlib.h>
-#include <time.h>
-
 #include <QtCore/QTimer>
 #include <QtCore/QFile>
 #include <QPointer>
@@ -47,6 +40,12 @@
 
 extern bool kio_resolve_local_urls; // from copyjob.cpp, abused here to save a symbol.
 
+static bool isHttpProtocol(const QString& protocol)
+{
+    return (protocol.startsWith(QLatin1String("webdav"), Qt::CaseInsensitive) ||
+            protocol.startsWith(QLatin1String("http"), Qt::CaseInsensitive));
+}
+
 namespace KIO
 {
     enum DeleteJobState {
@@ -55,14 +54,19 @@ namespace KIO
         DELETEJOB_STATE_DELETING_DIRS
     };
 
+    /*
+    static const char* const s_states[] = {
+        "DELETEJOB_STATE_STATING",
+        "DELETEJOB_STATE_DELETING_FILES",
+        "DELETEJOB_STATE_DELETING_DIRS"
+    };
+    */
+
     class DeleteJobPrivate: public KIO::JobPrivate
     {
     public:
         DeleteJobPrivate(const KUrl::List& src)
             : state( DELETEJOB_STATE_STATING )
-            , m_totalSize( 0 )
-            , m_processedSize( 0 )
-            , m_fileProcessedSize( 0 )
             , m_processedFiles( 0 )
             , m_processedDirs( 0 )
             , m_totalFilesDirs( 0 )
@@ -72,9 +76,6 @@ namespace KIO
         {
         }
         DeleteJobState state;
-        KIO::filesize_t m_totalSize;
-        KIO::filesize_t m_processedSize;
-        KIO::filesize_t m_fileProcessedSize;
         int m_processedFiles;
         int m_processedDirs;
         int m_totalFilesDirs;
@@ -92,10 +93,6 @@ namespace KIO
         void finishedStatPhase();
         void deleteNextFile();
         void deleteNextDir();
-        /**
-         * Forward signal from subjob
-         */
-        void slotProcessedSize( KJob*, qulonglong data_size );
         void slotReport();
         void slotStart();
         void slotEntries( KIO::Job*, const KIO::UDSEntryList& list );
@@ -151,7 +148,6 @@ void DeleteJobPrivate::slotReport()
 
    switch( state ) {
         case DELETEJOB_STATE_STATING:
-            q->setTotalAmount(KJob::Bytes, m_totalSize);
             q->setTotalAmount(KJob::Files, files.count());
             q->setTotalAmount(KJob::Directories, dirs.count());
             break;
@@ -176,7 +172,7 @@ void DeleteJobPrivate::slotEntries(KIO::Job* job, const UDSEntryList& list)
         const UDSEntry& entry = *it;
         const QString displayName = entry.stringValue( KIO::UDSEntry::UDS_NAME );
 
-        assert(!displayName.isEmpty());
+        Q_ASSERT(!displayName.isEmpty());
         if (displayName != ".." && displayName != ".")
         {
             KUrl url;
@@ -184,12 +180,10 @@ void DeleteJobPrivate::slotEntries(KIO::Job* job, const UDSEntryList& list)
             if ( !urlStr.isEmpty() )
                 url = urlStr;
             else {
-                url = ((SimpleJob *)job)->url(); // assumed to be a dir
+                url = static_cast<SimpleJob *>(job)->url(); // assumed to be a dir
                 url.addPath( displayName );
             }
 
-            m_totalSize += (KIO::filesize_t)entry.numberValue( KIO::UDSEntry::UDS_SIZE, 0 );
-
             //kDebug(7007) << displayName << "(" << url << ")";
             if ( entry.isLink() )
                 symlinks.append( url );
@@ -224,7 +218,7 @@ void DeleteJobPrivate::statNextSrc()
         // Fast path for KFileItems in directory views
         while(m_currentStat != m_srcList.end()) {
             m_currentURL = (*m_currentStat);
-            KFileItem cachedItem = KDirLister::cachedItemForUrl(m_currentURL);
+            const KFileItem cachedItem = KDirLister::cachedItemForUrl(m_currentURL);
             if (cachedItem.isNull())
                 break;
             //kDebug(7007) << "Found cached info about" << m_currentURL << "isDir=" << cachedItem.isDir() << "isLink=" << cachedItem.isLink();
@@ -239,7 +233,7 @@ void DeleteJobPrivate::statNextSrc()
             // (using a loop, instead of a huge recursion)
             while(m_currentStat != m_srcList.end() && (*m_currentStat).isLocalFile()) {
                 m_currentURL = (*m_currentStat);
-                QFileInfo fileInfo(m_currentURL.path());
+                QFileInfo fileInfo(m_currentURL.toLocalFile());
                 currentSourceStated(fileInfo.isDir(), fileInfo.isSymLink());
                 ++m_currentStat;
             }
@@ -249,7 +243,7 @@ void DeleteJobPrivate::statNextSrc()
             statNextSrc();
         } else {
             KIO::SimpleJob * job = KIO::stat( m_currentURL, StatJob::SourceSide, 0, KIO::HideProgressInfo );
-            Scheduler::scheduleJob(job);
+            Scheduler::setJobPriority(job, 1);
             //kDebug(7007) << "stat'ing" << m_currentURL;
             q->addSubjob(job);
         }
@@ -265,9 +259,10 @@ void DeleteJobPrivate::finishedStatPhase()
     slotReport();
     // Now we know which dirs hold the files we're going to delete.
     // To speed things up and prevent double-notification, we disable KDirWatch
-    // on those dirs temporarily (using KDirWatch::self, that's the instanced
+    // on those dirs temporarily (using KDirWatch::self, that's the instance
     // used by e.g. kdirlister).
-    for ( QSet<QString>::const_iterator it = m_parentDirs.constBegin() ; it != m_parentDirs.constEnd() ; ++it )
+    const QSet<QString>::const_iterator itEnd = m_parentDirs.constEnd();
+    for ( QSet<QString>::const_iterator it = m_parentDirs.constBegin() ; it != itEnd ; ++it )
         KDirWatch::self()->stopDirScan( *it );
     state = DELETEJOB_STATE_DELETING_FILES;
     deleteNextFile();
@@ -291,8 +286,12 @@ void DeleteJobPrivate::deleteNextFile()
             }
             // Normal deletion
             // If local file, try do it directly
-            if ( (*it).isLocalFile() && unlink( QFile::encodeName((*it).path()) ) == 0 ) {
-                //kdDebug(7007) << "DeleteJob deleted" << (*it).path();
+#ifdef Q_WS_WIN
+            if ( (*it).isLocalFile() && DeleteFileW( (LPCWSTR)(*it).toLocalFile().utf16() ) != 0 ) {
+#else
+            if ( (*it).isLocalFile() && unlink( QFile::encodeName((*it).toLocalFile()) ) == 0 ) {
+#endif
+                //kdDebug(7007) << "DeleteJob deleted" << (*it).toLocalFile();
                 job = 0;
                 m_processedFiles++;
                 if ( m_processedFiles % 300 == 1 || m_totalFilesDirs < 300) { // update progress info every 300 files
@@ -302,8 +301,11 @@ void DeleteJobPrivate::deleteNextFile()
             } else
             { // if remote - or if unlink() failed (we'll use the job's error handling in that case)
                 //kDebug(7007) << "calling file_delete on" << *it;
-                job = KIO::file_delete( *it, KIO::HideProgressInfo );
-                Scheduler::scheduleJob(job);
+                if (isHttpProtocol(it->protocol()))
+                  job = KIO::http_delete( *it, KIO::HideProgressInfo );
+                else
+                  job = KIO::file_delete( *it, KIO::HideProgressInfo );
+                Scheduler::setJobPriority(job, 1);
                 m_currentURL=(*it);
             }
             if ( isLink )
@@ -330,23 +332,22 @@ void DeleteJobPrivate::deleteNextDir()
             // Take first dir to delete out of list - last ones first !
             KUrl::List::iterator it = --dirs.end();
             // If local dir, try to rmdir it directly
-            if ( (*it).isLocalFile() && ::rmdir( QFile::encodeName((*it).path()) ) == 0 ) {
-
+#ifdef Q_WS_WIN
+            if ( (*it).isLocalFile() && RemoveDirectoryW( (LPCWSTR)(*it).toLocalFile().utf16() ) != 0 ) {
+#else
+            if ( (*it).isLocalFile() && ::rmdir( QFile::encodeName((*it).toLocalFile()) ) == 0 ) {
+#endif
                 m_processedDirs++;
                 if ( m_processedDirs % 100 == 1 ) { // update progress info every 100 dirs
                     m_currentURL = *it;
                     slotReport();
                 }
             } else {
-                SimpleJob* job;
-                if ( KProtocolManager::canDeleteRecursive( *it ) ) {
-                    // If the ioslave supports recursive deletion of a directory, then
-                    // we only need to send a single CMD_DEL command, so we use file_delete :)
-                    job = KIO::file_delete( *it, KIO::HideProgressInfo );
-                } else {
-                    job = KIO::rmdir( *it );
-                }
-                Scheduler::scheduleJob(job);
+                // Call rmdir - works for kioslaves with canDeleteRecursive too,
+                // CMD_DEL will trigger the recursive deletion in the slave.
+                SimpleJob* job = KIO::rmdir( *it );
+                job->addMetaData(QString::fromLatin1("recurse"), "true");
+                Scheduler::setJobPriority(job, 1);
                 dirs.erase(it);
                 q->addSubjob( job );
                 return;
@@ -356,8 +357,10 @@ void DeleteJobPrivate::deleteNextDir()
     }
 
     // Re-enable watching on the dirs that held the deleted files
-    for (QSet<QString>::const_iterator it = m_parentDirs.constBegin() ; it != m_parentDirs.constEnd() ; ++it)
+    const QSet<QString>::const_iterator itEnd = m_parentDirs.constEnd();
+    for (QSet<QString>::const_iterator it = m_parentDirs.constBegin() ; it != itEnd ; ++it) {
         KDirWatch::self()->restartDirScan( *it );
+    }
 
     // Finished - tell the world
     if ( !m_srcList.isEmpty() )
@@ -370,28 +373,6 @@ void DeleteJobPrivate::deleteNextDir()
     q->emitResult();
 }
 
-// Note: I don't think this slot is connected to anywhere! -thiago
-void DeleteJobPrivate::slotProcessedSize( KJob*, qulonglong data_size )
-{
-   Q_Q(DeleteJob);
-   // Note: this is the same implementation as CopyJob::slotProcessedSize but
-   // it's different from FileCopyJob::slotProcessedSize - which is why this
-   // is not in Job.
-
-   m_fileProcessedSize = data_size;
-   q->setProcessedAmount(KJob::Bytes, m_processedSize + m_fileProcessedSize);
-
-   //kDebug(7007) << (unsigned int) (m_processedSize + m_fileProcessedSize);
-
-   q->setProcessedAmount(KJob::Bytes, m_processedSize + m_fileProcessedSize);
-
-   // calculate percents
-   if ( m_totalSize == 0 )
-      q->setPercent( 100 );
-   else
-      q->setPercent( (unsigned long)(( (float)(m_processedSize + m_fileProcessedSize) / (float)m_totalSize ) * 100.0) );
-}
-
 void DeleteJobPrivate::currentSourceStated(bool isDir, bool isLink)
 {
     Q_Q(DeleteJob);
@@ -400,16 +381,19 @@ void DeleteJobPrivate::currentSourceStated(bool isDir, bool isLink)
         // Add toplevel dir in list of dirs
         dirs.append( url );
         if (url.isLocalFile()) {
-            const QString parentDir = url.path(KUrl::RemoveTrailingSlash);
-            m_parentDirs.insert(parentDir);
+            // We are about to delete this dir, no need to watch it
+            // Maybe we should ask kdirwatch to remove all watches recursively?
+            // But then there would be no feedback (things disappearing progressively) during huge deletions
+            KDirWatch::self()->stopDirScan(url.toLocalFile(KUrl::RemoveTrailingSlash));
         }
         if (!KProtocolManager::canDeleteRecursive(url)) {
             //kDebug(7007) << url << "is a directory, let's list it";
             ListJob *newjob = KIO::listRecursive(url, KIO::HideProgressInfo);
+            newjob->addMetaData("details", "0");
             newjob->setUnrestricted(true); // No KIOSK restrictions
-            Scheduler::scheduleJob(newjob);
-            QObject::connect(newjob, SIGNAL(entries(KIO::Job*, const KIO::UDSEntryList&)),
-                             q, SLOT(slotEntries(KIO::Job*,const KIO::UDSEntryList&)));
+            Scheduler::setJobPriority(newjob, 1);
+            QObject::connect(newjob, SIGNAL(entries(KIO::Job*,KIO::UDSEntryList)),
+                             q, SLOT(slotEntries(KIO::Job*,KIO::UDSEntryList)));
             q->addSubjob(newjob);
             // Note that this listing job will happen in parallel with other stat jobs.
         }
@@ -421,10 +405,10 @@ void DeleteJobPrivate::currentSourceStated(bool isDir, bool isLink)
             //kDebug(7007) << "Target is a file";
             files.append(url);
         }
-        if (url.isLocalFile()) {
-            const QString parentDir = url.directory(KUrl::ObeyTrailingSlash);
-            m_parentDirs.insert(parentDir);
-        }
+    }
+    if (url.isLocalFile()) {
+        const QString parentDir = url.directory(KUrl::IgnoreTrailingSlash);
+        m_parentDirs.insert(parentDir);
     }
 }
 
@@ -473,7 +457,7 @@ void DeleteJob::slotResult( KJob *job )
             return;
         }
         removeSubjob( job );
-        assert( !hasSubjobs() );
+        Q_ASSERT( !hasSubjobs() );
         d->m_processedFiles++;
 
         d->deleteNextFile();
@@ -485,7 +469,7 @@ void DeleteJob::slotResult( KJob *job )
             return;
         }
         removeSubjob( job );
-        assert( !hasSubjobs() );
+        Q_ASSERT( !hasSubjobs() );
         d->m_processedDirs++;
         //emit processedAmount( this, KJob::Directories, d->m_processedDirs );
         //emitPercent( d->m_processedFiles + d->m_processedDirs, d->m_totalFilesDirs );
@@ -493,7 +477,7 @@ void DeleteJob::slotResult( KJob *job )
         d->deleteNextDir();
         break;
     default:
-        assert(0);
+        Q_ASSERT(0);
     }
 }
 
