@@ -35,18 +35,24 @@
 #include "operations.h"
 #include "debugger.h"
 #include "PropertyNameArray.h"
+#include "commonunicode.h"
 
 #include <stdio.h>
 #include <errno.h>
 #include <stdlib.h>
 #include <assert.h>
 #include <string.h>
+#ifndef _WIN32
 #include "wtf/DisallowCType.h"
+#endif
 #include "wtf/ASCIICType.h"
 #include "bytecode/machine.h"
+#include <string>
 
 using namespace WTF;
 
+//#define KJS_VERBOSE
+
 namespace KJS {
 
 // ----------------------------- FunctionImp ----------------------------------
@@ -93,11 +99,42 @@ void FunctionImp::initialCompile(ExecState* newExec)
     body->compile(FunctionCode, newExec->dynamicInterpreter()->debugger() ? Debug : Release);
 }
 
+
+#ifdef KJS_VERBOSE
+static int           callDepth;
+static std::string   callIndent;
+
+static const char* ind()
+{
+    callIndent = "";
+    for (int i = 0; i < callDepth; ++i)
+        callIndent += "     ";
+    return callIndent.c_str();
+}
+
+// Multiline print adding indentation
+static void printInd(const char* str)
+{
+    fprintf(stderr, "%s", ind());
+    for (const char* c = str; *c; ++c) {
+        if (*c != '\n')
+            fprintf(stderr, "%c", *c);
+        else
+            fprintf(stderr, "\n%s", ind());
+    }
+}
+
+#endif
+
 JSValue* FunctionImp::callAsFunction(ExecState* exec, JSObject* thisObj, const List& args)
 {
   assert(thisObj);
 
-  Debugger* dbg = exec->dynamicInterpreter()->debugger();  
+#ifdef KJS_VERBOSE
+  ++callDepth;
+#endif
+
+  Debugger* dbg = exec->dynamicInterpreter()->debugger();
 
   // enter a new execution context
   FunctionExecState newExec(exec->dynamicInterpreter(), thisObj, body.get(), exec, this);
@@ -131,14 +168,6 @@ JSValue* FunctionImp::callAsFunction(ExecState* exec, JSObject* thisObj, const L
   activation->setup(&newExec, this, &args, stackSpace);
   activation->tearOffNeededSlot() = body->tearOffAtEnd();
 
-  if (dbg) {
-    bool cont = dbg->enterContext(&newExec, body->sourceId(), body->firstLine(), this, args);
-    if (!cont) {
-      dbg->imp()->abort();
-      return jsUndefined();
-    }
-  }
-
   newExec.initLocalStorage(stackSpace, regs);
 
   JSValue* result = Machine::runBlock(&newExec, body->code(), exec);
@@ -160,25 +189,15 @@ JSValue* FunctionImp::callAsFunction(ExecState* exec, JSObject* thisObj, const L
   exec->dynamicInterpreter()->stackFree(stackSize);
 
 #ifdef KJS_VERBOSE
+  fprintf(stderr, "%s", ind());
   if (exec->exception())
     printInfo(exec,"throwing", exec->exception());
   else
     printInfo(exec,"returning", result);
+
+  --callDepth;
 #endif
 
-  // The debugger may have been deallocated by now if the WebFrame
-  // we were running in has been destroyed, so refetch it.
-  // See http://bugs.webkit.org/show_bug.cgi?id=9477
-  dbg = exec->dynamicInterpreter()->debugger();
-
-  if (dbg) {
-    int cont = dbg->exitContext(&newExec, body->sourceId(), body->lastLine(), this);
-    if (!cont) {
-      dbg->imp()->abort();
-      return jsUndefined();
-    }
-  }
-
   return result;
 }
 
@@ -339,7 +358,7 @@ JSObject *FunctionImp::construct(ExecState *exec, const List &args)
 IndexToNameMap::IndexToNameMap(FunctionImp *func, const List &args)
 {
   _map = new Identifier[args.size()];
-  this->size = args.size();
+  this->_size = args.size();
 
   size_t i = 0;
   ListIterator iterator = args.begin();
@@ -354,12 +373,12 @@ IndexToNameMap::~IndexToNameMap() {
 bool IndexToNameMap::isMapped(const Identifier &index) const
 {
   bool indexIsNumber;
-  int indexAsNumber = index.toUInt32(&indexIsNumber);
+  int indexAsNumber = index.toStrictUInt32(&indexIsNumber);
 
   if (!indexIsNumber)
     return false;
 
-  if (indexAsNumber >= size)
+  if (indexAsNumber >= _size)
     return false;
 
   if (_map[indexAsNumber].isNull())
@@ -371,13 +390,18 @@ bool IndexToNameMap::isMapped(const Identifier &index) const
 void IndexToNameMap::unMap(const Identifier &index)
 {
   bool indexIsNumber;
-  int indexAsNumber = index.toUInt32(&indexIsNumber);
+  int indexAsNumber = index.toStrictUInt32(&indexIsNumber);
 
-  assert(indexIsNumber && indexAsNumber < size);
+  assert(indexIsNumber && indexAsNumber < _size);
 
   _map[indexAsNumber] = CommonIdentifiers::shared()->nullIdentifier;;
 }
 
+int IndexToNameMap::size() const
+{
+    return _size;
+}
+
 Identifier& IndexToNameMap::operator[](int index)
 {
   return _map[index];
@@ -386,9 +410,9 @@ Identifier& IndexToNameMap::operator[](int index)
 Identifier& IndexToNameMap::operator[](const Identifier &index)
 {
   bool indexIsNumber;
-  int indexAsNumber = index.toUInt32(&indexIsNumber);
+  int indexAsNumber = index.toStrictUInt32(&indexIsNumber);
 
-  assert(indexIsNumber && indexAsNumber < size);
+  assert(indexIsNumber && indexAsNumber < _size);
 
   return (*this)[indexAsNumber];
 }
@@ -457,6 +481,25 @@ bool Arguments::deleteProperty(ExecState *exec, const Identifier &propertyName)
   }
 }
 
+void Arguments::getOwnPropertyNames(ExecState* exec, PropertyNameArray& propertyNames, PropertyMap::PropertyMode mode)
+{
+    unsigned int length = indexToNameMap.size();
+    unsigned attr;
+    for (unsigned int i = 0; i < length; ++i) {
+        attr = 0;
+        Identifier ident = Identifier::from(i);
+
+        if (indexToNameMap.isMapped(ident) &&
+            _activationObject->getPropertyAttributes(indexToNameMap[ident], attr)) {
+            if (PropertyMap::checkEnumerable(attr, mode)) {
+                propertyNames.add(ident);
+            }
+        }
+    }
+
+    JSObject::getOwnPropertyNames(exec, propertyNames, mode);
+}
+
 // ------------------------------ ActivationImp --------------------------------
 
 const ClassInfo ActivationImp::info = {"Activation", 0, 0, 0};
@@ -488,23 +531,43 @@ void ActivationImp::setup(ExecState* exec, FunctionImp *function,
 
     // Pass in the parameters (ECMA 10.1.3q)
 #ifdef KJS_VERBOSE
-    fprintf(stderr, "---------------------------------------------------\n"
-            "processing parameters for %s call\n",
+    fprintf(stderr, "%s---------------------------------------------------\n"
+            "%sprocessing parameters for %s call\n", ind(), ind(), 
             function->functionName().isEmpty() ? "(internal)" : function->functionName().ascii());
 #endif
-    size_t numParams = body->numParams();
-    for (size_t pos = 0; pos < numParams; ++pos) {
+    size_t numParams   = body->numParams();
+    size_t numPassedIn = min(numParams, static_cast<size_t>(arguments->size()));
+
+    size_t pos = 0; 
+    for (; pos < numPassedIn; ++pos) {
         size_t symNum = pos + ActivationImp::NumReservedSlots;
-        JSValue* v = (*arguments)[pos];
+        JSValue* v = arguments->atUnchecked(pos);
 
         entries[symNum].val.valueVal = v;
 
 #ifdef KJS_VERBOSE
-        fprintf(stderr, "setting parameter %s", body->paramName(pos).ascii());
+        fprintf(stderr, "%s setting parameter %s", ind(), body->paramName(pos).ascii());
         printInfo(exec, "to", v);
 #endif
     }
 
+    for (; pos < numParams; ++pos) {
+        size_t symNum = pos + ActivationImp::NumReservedSlots;
+        entries[symNum].val.valueVal = jsUndefined();
+
+#ifdef KJS_VERBOSE
+        fprintf(stderr, "%s setting parameter %s to undefined (not passed in)", ind(), body->paramName(pos).ascii());
+#endif
+    }
+    
+#ifdef KJS_VERBOSE
+    fprintf(stderr, "\n%s---------------------------------\n", ind());
+    fprintf(stderr, "%sBody:\n", ind());
+    fprintf(stderr, "%s---------------------------------\n", ind());
+    printInd(body->toString().ascii());
+    fprintf(stderr, "\n%s---------------------------------\n\n", ind());
+#endif
+
     // Initialize the rest of the locals to 'undefined'
     for (size_t pos = numParams + ActivationImp::NumReservedSlots; pos < total; ++pos)
         entries[pos].val.valueVal = jsUndefined();
@@ -684,41 +747,6 @@ static JSValue *decode(ExecState *exec, const List &args, const char *do_not_une
   return jsString(s);
 }
 
-static bool isStrWhiteSpace(unsigned short c)
-{
-    switch (c) {
-        case 0x0009:
-        case 0x000A:
-        case 0x000B:
-        case 0x000C:
-        case 0x000D:
-        case 0x2028:
-        case 0x2029:
-        // Unicode category Zs
-        case 0x0020:  // SPACE
-        case 0x00A0:  // NO-BREAK SPACE
-        case 0x1680:  // OGHAM SPACE MARK
-        case 0x180E:  // MONGOLIAN VOWEL SEPARATOR
-        case 0x2000:  // EN QUAD
-        case 0x2001:  // EM QUAD
-        case 0x2002:  // EN SPACE
-        case 0x2003:  // EM SPACE
-        case 0x2004:  // THREE-PER-EM SPACE
-        case 0x2005:  // FOUR-PER-EM SPACE
-        case 0x2006:  // SIX-PER-EM SPACE
-        case 0x2007:  // FIGURE SPACE
-        case 0x2008:  // PUNCTUATION SPACE
-        case 0x2009:  // THIN SPACE
-        case 0x200A:  // HAIR SPACE
-        case 0x202F:  // NARROW NO-BREAK SPACE
-        case 0x205F:  // MEDIUM MATHEMATICAL SPACE
-        case 0x3000:  // IDEOGRAPHIC SPACE
-            return true;
-        default:
-            return false;
-    }
-}
-
 static int parseDigit(unsigned short c, int radix)
 {
     int digit = -1;
@@ -763,7 +791,7 @@ static double parseInt(const UString &s, int radix)
     int length = s.size();
     int p = 0;
 
-    while (p < length && isStrWhiteSpace(s[p].uc)) {
+    while (p < length && CommonUnicode::isStrWhiteSpace(s[p].uc)) {
         ++p;
     }
 
@@ -781,6 +809,7 @@ static double parseInt(const UString &s, int radix)
         radix = 16;
         p += 2;
     } else if (radix == 0) {
+        // ECMAscript test262 S15.1.2.2_A5.1_T1 says we should no longer accept octal. To fix remove next 3 lines.
         if (p < length && s[p] == '0')
             radix = 8;
         else
@@ -822,7 +851,7 @@ static double parseFloat(const UString &s)
     // Need to skip any whitespace and then one + or - sign.
     int length = s.size();
     int p = 0;
-    while (p < length && isStrWhiteSpace(s[p].uc)) {
+    while (p < length && CommonUnicode::isStrWhiteSpace(s[p].uc)) {
         ++p;
     }
     if (p < length && (s[p] == '+' || s[p] == '-')) {
@@ -1013,3 +1042,5 @@ JSValue *GlobalFuncImp::callAsFunction(ExecState *exec, JSObject * /*thisObj*/,
 }
 
 } // namespace
+
+// kate: indent-width 4; replace-tabs on; tab-width 4; space-indent on;
