@@ -258,8 +258,8 @@ void CoreEngine::loadEntries(Provider *provider)
                     SIGNAL(signalEntriesFailed()),
                     SLOT(slotEntriesFailed()));
             connect(entry_loader,
-                    SIGNAL(signalProgress(KJob*, unsigned long)),
-                    SLOT(slotProgress(KJob*, unsigned long)));
+                    SIGNAL(signalProgress(KJob*,ulong)),
+                    SLOT(slotProgress(KJob*,ulong)));
 
             entry_loader->load(provider, feed);
         }
@@ -291,8 +291,8 @@ void CoreEngine::downloadPreview(Entry *entry)
             SIGNAL(result(KJob*)),
             SLOT(slotPreviewResult(KJob*)));
     connect(job,
-            SIGNAL(progress(KJob*, unsigned long)),
-            SLOT(slotProgress(KJob*, unsigned long)));
+            SIGNAL(progress(KJob*,ulong)),
+            SLOT(slotProgress(KJob*,ulong)));
 
     m_entry_jobs[job] = entry;
 }
@@ -331,8 +331,8 @@ void CoreEngine::downloadPayload(Entry *entry)
             SIGNAL(result(KJob*)),
             SLOT(slotPayloadResult(KJob*)));
     connect(job,
-            SIGNAL(percent(KJob*, unsigned long)),
-            SLOT(slotProgress(KJob*, unsigned long)));
+            SIGNAL(percent(KJob*,ulong)),
+            SLOT(slotProgress(KJob*,ulong)));
 
     m_entry_jobs[job] = entry;
 }
@@ -1407,12 +1407,16 @@ bool CoreEngine::install(const QString &payloadfile)
         }
         if (!m_installation->installPath().isEmpty()) {
 #if defined(Q_WS_WIN)
+#ifndef _WIN32_WCE
             WCHAR wPath[MAX_PATH+1];
             if ( SHGetFolderPathW(NULL, CSIDL_APPDATA, NULL, SHGFP_TYPE_CURRENT, wPath) == S_OK) {
                 installdir = QString::fromUtf16((const ushort *) wPath) + QLatin1Char('/') + m_installation->installPath() + QLatin1Char('/');
             } else {
+#endif
                 installdir =  QDir::home().path() + QLatin1Char('/') + m_installation->installPath() + QLatin1Char('/');
+#ifndef _WIN32_WCE
             }
+#endif
 #else
             installdir = QDir::home().path() + '/' + m_installation->installPath() + '/';
 #endif
